@@ -6,9 +6,9 @@
    It may be used under the terms of the GNU General Public License v2.
    For full terms see the file COPYING file or visit http://www.gnu.org/licenses/gpl-2.0.html
  */
- 
-#include "hb.h"
-#include "lang.h"
+
+#include "handbrake/handbrake.h"
+#include "handbrake/lang.h"
 
 struct hb_batch_s
 {
