@@ -13,8 +13,8 @@
  *
  * Note that the SRT sub generation from CC could be useful for mkv subs.
  */
-#include "hb.h"
-#include "deccc608sub.h"
+#include "handbrake/handbrake.h"
+#include "handbrake/deccc608sub.h"
 
 #define SSA_PREAMBLE_LEN 24
 /*
@@ -297,7 +297,7 @@ static int get_char_in_utf8(unsigned char *buffer, unsigned char c)
             *buffer = 0xc3;
             *(buffer+1) = 0x9a;
             return 2;
-        case 0x94: // capital letter U with diaresis
+        case 0x94: // capital letter U with diaeresis
             *buffer = 0xc3;
             *(buffer+1) = 0x9c;
             return 2;
@@ -333,10 +333,10 @@ static int get_char_in_utf8(unsigned char *buffer, unsigned char c)
         case 0x9d: // Full stop (.)
             *buffer = 0x2e;
             return 1;
-        case 0x9e: // Quoatation mark
+        case 0x9e: // Quotation mark
             *buffer = 0x22;
             return 1;
-        case 0x9f: // Quoatation mark
+        case 0x9f: // Quotation mark
             *buffer = 0x22;
             return 1;
         case 0xa0: // uppercase A, grave accent
@@ -359,11 +359,11 @@ static int get_char_in_utf8(unsigned char *buffer, unsigned char c)
             *buffer = 0xc3;
             *(buffer+1) = 0x8a;
             return 2;
-        case 0xa5: // capital letter E with diaresis
+        case 0xa5: // capital letter E with diaeresis
             *buffer = 0xc3;
             *(buffer+1) = 0x8b;
             return 2;
-        case 0xa6: // lowercase letter e with diaresis
+        case 0xa6: // lowercase letter e with diaeresis
             *buffer = 0xc3;
             *(buffer+1) = 0xab;
             return 2;
@@ -371,11 +371,11 @@ static int get_char_in_utf8(unsigned char *buffer, unsigned char c)
             *buffer = 0xc3;
             *(buffer+1) = 0x8e;
             return 2;
-        case 0xa8: // uppercase I, with diaresis
+        case 0xa8: // uppercase I, with diaeresis
             *buffer = 0xc3;
             *(buffer+1) = 0x8f;
             return 2;
-        case 0xa9: // lowercase i, with diaresis
+        case 0xa9: // lowercase i, with diaeresis
             *buffer = 0xc3;
             *(buffer+1) = 0xaf;
             return 2;
@@ -877,8 +877,8 @@ static int write_cc_buffer_as_ssa(struct eia608_screen *data,
     safe_y = 0.1 * wb->height;
     min_safe_x = 0.025 * cropped_width;
     min_safe_y = 0.025 * cropped_height;
-    cell_height = (wb->height - 2 * safe_y) / 16; 
-    cell_width  = (wb->width  - 2 * safe_x) / screen_columns; 
+    cell_height = (wb->height - 2 * safe_y) / 16;
+    cell_width  = (wb->width  - 2 * safe_x) / screen_columns;
 
     char *pos;
     int y, x, top;
@@ -1365,7 +1365,7 @@ static void handle_command(unsigned char c1, const unsigned char c2,
             }
             erase_memory (wb,1);
 
-            // the last pts is the time to remove the previously 
+            // the last pts is the time to remove the previously
             // displayed CC from the display
             wb->data608->current_visible_start_ms = wb->last_pts;
             wb->data608->current_visible_scr_sequence = wb->last_scr_sequence;
@@ -1513,7 +1513,7 @@ static void handle_pac(unsigned char c1, unsigned char c2, struct s_write *wb)
                 font_text[wb->data608->font]);
 
     // CC spec says to the preferred method to handle a roll-up base row
-    // that causes the display to scroll off the top of the screen is to 
+    // that causes the display to scroll off the top of the screen is to
     // adjust the base row down.
     int keep_lines;
     switch (wb->data608->mode)
