@@ -12,9 +12,9 @@
 #include <string.h>
 #include <iconv.h>
 #include <errno.h>
-#include "hb.h"
-#include "colormap.h"
-#include "decsrtsub.h"
+#include "handbrake/handbrake.h"
+#include "handbrake/colormap.h"
+#include "handbrake/decsrtsub.h"
 
 struct start_and_end {
     unsigned long start, end;
@@ -326,7 +326,7 @@ static int get_line( hb_work_private_t * pv, char *buf, int size )
     int i;
     char c;
 
-    // clear remnants of the previous line before progessing a new one
+    // clear remnants of the previous line before processing a new one
     memset(buf, '\0', size);
 
     /* Find newline in converted UTF-8 buffer */
@@ -493,7 +493,10 @@ static hb_buffer_t *srt_read( hb_work_private_t *pv )
                 uint64_t stop_time = ( pv->current_entry.stop +
                                        pv->subtitle->config.offset ) * 90;
 
-                if( !( start_time >= pv->start_time && stop_time < pv->stop_time ) )
+                // Drop subtitles that end before the start time
+                // or start after the stop time
+                if (stop_time  <= pv->start_time ||
+                    start_time >= pv->stop_time)
                 {
                     hb_deep_log( 3, "Discarding SRT at time start %"PRId64", stop %"PRId64, start_time, stop_time);
                     memset( &pv->current_entry, 0, sizeof( srt_entry_t ) );
@@ -501,6 +504,14 @@ static hb_buffer_t *srt_read( hb_work_private_t *pv )
                     pv->current_state = k_state_timecode;
                     continue;
                 }
+                if (start_time < pv->start_time)
+                {
+                    start_time = pv->start_time;
+                }
+                if (stop_time > pv->stop_time)
+                {
+                    stop_time = pv->stop_time;
+                }
 
                 for (q = p = pv->current_entry.text; *p != '\0'; p++)
                 {
