@@ -9,7 +9,7 @@
 
 /*
  * Decoder for DVD bitmap subtitles, also known as "VOB subtitles" within the HandBrake source code.
- * 
+ *
  * Input format of the subtitle packets is described here:
  *   http://sam.zoy.org/writings/dvd/subtitles/
  *
@@ -27,7 +27,7 @@
  *   }
  */
 
-#include "hb.h"
+#include "handbrake/handbrake.h"
 
 struct hb_work_private_s
 {
@@ -68,7 +68,7 @@ int decsubInit( hb_work_object_t * w, hb_job_t * job )
 
     pv->job = job;
     pv->pts = 0;
-    
+
     // Warn if the input color palette is empty
     pv->palette_set = w->subtitle->palette_set;
     if ( pv->palette_set )
@@ -168,10 +168,8 @@ int decsubWork( hb_work_object_t * w, hb_buffer_t ** buf_in,
         /* We got a complete subtitle, decode it */
         *buf_out = Decode( w );
 
-        if( buf_out && *buf_out )
-        {
+        if ( *buf_out != NULL )
             (*buf_out)->s.id = in->s.id;
-        }
 
         /* Wait for the next one */
         pv->size_sub = 0;
@@ -182,7 +180,7 @@ int decsubWork( hb_work_object_t * w, hb_buffer_t ** buf_in,
         {
             // If we don't get a valid next timestamp, use the stop time
             // of the current sub as the start of the next.
-            // This can happen if reader invalidates timestamps while 
+            // This can happen if reader invalidates timestamps while
             // waiting for an audio to update the SCR.
             pv->pts                  = pv->pts_stop;
             pv->current_scr_sequence = pv->scr_sequence;
@@ -630,7 +628,7 @@ static hb_buffer_t * Decode( hb_work_object_t * w )
     if ( ParseControls( w ) )
     {
         /*
-         * Coudln't parse the info
+         * Couldn't parse the info
          */
         hb_deep_log( 2, "decvobsub: Could not parse info!" );
 
@@ -678,6 +676,7 @@ static hb_buffer_t * Decode( hb_work_object_t * w )
         for( col = 0; col < pv->width; col += code >> 2 )
         {
             uint8_t * lum, * alpha,  * chromaU, * chromaV;
+            int       idx, len;
 
             code = 0;
             GET_NEXT_NIBBLE;
@@ -699,19 +698,22 @@ static hb_buffer_t * Decode( hb_work_object_t * w )
                 }
             }
 
-            lum   = buf_raw;
-            alpha = lum + pv->width * pv->height;
+            lum     = buf_raw;
+            alpha   = lum + pv->width * pv->height;
             chromaU = alpha + pv->width * pv->height;
             chromaV = chromaU + pv->width * pv->height;
+            idx     = code & 3;
+            len     = code >> 2;
+            // Protect against malformed VOBSUB with invalid run length
+            if (len > pv->width - col)
+            {
+                len = pv->width - col;
+            }
 
-            memset( lum + line * pv->width + col,
-                    pv->lum[code & 3], code >> 2 );
-            memset( alpha + line * pv->width + col,
-                    pv->alpha[code & 3], code >> 2 );
-            memset( chromaU + line * pv->width + col,
-                    pv->chromaU[code & 3], code >> 2 );
-            memset( chromaV + line * pv->width + col,
-                    pv->chromaV[code & 3], code >> 2 );
+            memset( lum     + line * pv->width + col, pv->lum[idx],     len );
+            memset( alpha   + line * pv->width + col, pv->alpha[idx],   len );
+            memset( chromaU + line * pv->width + col, pv->chromaU[idx], len );
+            memset( chromaV + line * pv->width + col, pv->chromaV[idx], len );
         }
 
         /* Byte-align */
