@@ -7,8 +7,8 @@
    For full terms see the file COPYING file or visit http://www.gnu.org/licenses/gpl-2.0.html
  */
 
-#include "hb.h"
-#include "hbffmpeg.h"
+#include "handbrake/handbrake.h"
+#include "handbrake/hbffmpeg.h"
 
 struct hb_work_private_s
 {
@@ -249,13 +249,13 @@ static int encavcodecaInit(hb_work_object_t *w, hb_job_t *job)
                        context->channel_layout, 0);
         av_opt_set_int(pv->swresample, "out_channel_layout",
                        context->channel_layout, 0);
+        av_opt_set_int(pv->swresample, "in_sample_rate",
+                       context->sample_rate, 0);
+        av_opt_set_int(pv->swresample, "out_sample_rate",
+                       context->sample_rate, 0);
         if (hb_audio_dither_is_supported(audio->config.out.codec))
         {
             // dithering needs the sample rate
-            av_opt_set_int(pv->swresample, "in_sample_rate",
-                           context->sample_rate, 0);
-            av_opt_set_int(pv->swresample, "out_sample_rate",
-                           context->sample_rate, 0);
             av_opt_set_int(pv->swresample, "dither_method",
                            audio->config.out.dither_method, 0);
         }
