@@ -7,9 +7,12 @@
    For full terms see the file COPYING file or visit http://www.gnu.org/licenses/gpl-2.0.html
  */
 
-#include "hb.h"
-#ifdef USE_QSV
-#include "qsv_libav.h"
+#include "libavcodec/avcodec.h"
+
+#include "handbrake/handbrake.h"
+#if HB_PROJECT_FEATURE_QSV
+#include "handbrake/qsv_libav.h"
+#include "handbrake/qsv_common.h"
 #endif
 
 #ifndef SYS_DARWIN
@@ -55,7 +58,7 @@ struct hb_fifo_s
 };
 
 #if defined(HB_FIFO_DEBUG)
-static hb_fifo_t fifo_list = 
+static hb_fifo_t fifo_list =
 {
     .next = NULL
 };
@@ -295,7 +298,7 @@ void hb_buffer_pool_free( void )
             if( b->data )
             {
                 freed += b->alloc;
-                free(b->data);
+                av_free(b->data);
             }
             free( b );
             count++;
@@ -386,7 +389,7 @@ hb_buffer_t * hb_buffer_init_internal( int size )
      */
     if( !( b = calloc( sizeof( hb_buffer_t ), 1 ) ) )
     {
-        hb_log( "out of memory" );
+        hb_error( "out of memory" );
         return NULL;
     }
 
@@ -395,18 +398,10 @@ hb_buffer_t * hb_buffer_init_internal( int size )
 
     if (size)
     {
-#if defined( SYS_DARWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || defined( SYS_MINGW )
-        b->data  = malloc( b->alloc );
-#elif defined( SYS_CYGWIN )
-        /* FIXME */
-        b->data  = malloc( b->alloc + 17 );
-#else
-        b->data  = memalign( 16, b->alloc );
-#endif
-
+        b->data = av_malloc(b->alloc);
         if( !b->data )
         {
-            hb_log( "out of memory" );
+            hb_error( "out of memory" );
             free( b );
             return NULL;
         }
@@ -445,13 +440,25 @@ void hb_buffer_realloc( hb_buffer_t * b, int size )
 {
     if ( size > b->alloc || b->data == NULL )
     {
-        uint32_t orig = b->data != NULL ? b->alloc : 0;
-        hb_fifo_t *buffer_pool = size_to_pool(size);
+        uint8_t   * tmp;
+        uint32_t    orig = b->data != NULL ? b->alloc : 0;
+        hb_fifo_t * buffer_pool = size_to_pool(size);
+
         if (buffer_pool != NULL)
         {
             size = buffer_pool->buffer_size;
         }
-        b->data  = realloc( b->data, size );
+        tmp = av_malloc(size);
+        if (tmp == NULL)
+        {
+            return;
+        }
+        if (b->data != NULL)
+        {
+            memcpy(tmp, b->data, b->alloc);
+            av_free(b->data);
+        }
+        b->data  = tmp;
         b->alloc = size;
 
         hb_lock(buffers.lock);
@@ -492,7 +499,7 @@ hb_buffer_t * hb_buffer_dup( const hb_buffer_t * src )
             hb_buffer_init_planes( buf );
     }
 
-#ifdef USE_QSV
+#if HB_PROJECT_FEATURE_QSV
     memcpy(&buf->qsv_details, &src->qsv_details, sizeof(src->qsv_details));
 #endif
 
@@ -516,70 +523,52 @@ int hb_buffer_copy(hb_buffer_t * dst, const hb_buffer_t * src)
     return 0;
 }
 
-static void hb_buffer_init_planes_internal( hb_buffer_t * b, uint8_t * has_plane )
+void hb_buffer_init_planes(hb_buffer_t * b)
 {
-    uint8_t * plane = b->data;
-    int p;
+    uint8_t * data = b->data;
+    int       pp;
 
-    for( p = 0; p < 4; p++ )
+    for( pp = 0; pp <= b->f.max_plane; pp++ )
     {
-        if ( has_plane[p] )
-        {
-            b->plane[p].data = plane;
-            b->plane[p].stride = hb_image_stride( b->f.fmt, b->f.width, p );
-            b->plane[p].height_stride = hb_image_height_stride( b->f.fmt, b->f.height, p );
-            b->plane[p].width  = hb_image_width( b->f.fmt, b->f.width, p );
-            b->plane[p].height = hb_image_height( b->f.fmt, b->f.height, p );
-            b->plane[p].size   = b->plane[p].stride * b->plane[p].height_stride;
-            plane += b->plane[p].size;
-        }
+        b->plane[pp].data = data;
+        b->plane[pp].stride        = hb_image_stride(b->f.fmt, b->f.width, pp);
+        b->plane[pp].height_stride = hb_image_height_stride(b->f.fmt,
+                                                            b->f.height, pp);
+        b->plane[pp].width         = hb_image_width(b->f.fmt, b->f.width, pp);
+        b->plane[pp].height        = hb_image_height(b->f.fmt, b->f.height, pp);
+        b->plane[pp].size          = b->plane[pp].stride *
+                                     b->plane[pp].height_stride;
+        data                      += b->plane[pp].size;
     }
 }
 
-void hb_buffer_init_planes( hb_buffer_t * b )
-{
-    const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(b->f.fmt);
-    int p;
-
-    if (desc == NULL)
-    {
-        return;
-    }
-
-    uint8_t has_plane[4] = {0,};
-
-    for( p = 0; p < 4; p++ )
-    {
-        has_plane[desc->comp[p].plane] = 1;
-    }
-    hb_buffer_init_planes_internal( b, has_plane );
-}
-
 // this routine gets a buffer for an uncompressed picture
 // with pixel format pix_fmt and dimensions width x height.
 hb_buffer_t * hb_frame_buffer_init( int pix_fmt, int width, int height )
 {
-    const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(pix_fmt);
-    hb_buffer_t * buf;
-    int p;
-    uint8_t has_plane[4] = {0,};
+    const AVPixFmtDescriptor * desc = av_pix_fmt_desc_get(pix_fmt);
+    hb_buffer_t              * buf;
+    uint8_t                    has_plane[4] = {0,};
+    int                        ii, pp, max_plane = 0;
 
     if (desc == NULL)
     {
         return NULL;
     }
-    for( p = 0; p < 4; p++ )
-    {
-        has_plane[desc->comp[p].plane] = 1;
-    }
 
     int size = 0;
-    for( p = 0; p < 4; p++ )
+    for (ii = 0; ii < desc->nb_components; ii++)
     {
-        if ( has_plane[p] )
+        pp    = desc->comp[ii].plane;
+        if (pp > max_plane)
         {
-            size += hb_image_stride( pix_fmt, width, p ) * 
-                    hb_image_height_stride( pix_fmt, height, p );
+            max_plane = pp;
+        }
+        if (!has_plane[pp])
+        {
+            has_plane[pp] = 1;
+            size += hb_image_stride( pix_fmt, width, pp ) *
+                    hb_image_height_stride( pix_fmt, height, pp );
         }
     }
 
@@ -588,12 +577,13 @@ hb_buffer_t * hb_frame_buffer_init( int pix_fmt, int width, int height )
     if( buf == NULL )
         return NULL;
 
+    buf->f.max_plane = max_plane;
     buf->s.type = FRAME_BUF;
     buf->f.width = width;
     buf->f.height = height;
     buf->f.fmt = pix_fmt;
 
-    hb_buffer_init_planes_internal( buf, has_plane );
+    hb_buffer_init_planes(buf);
     return buf;
 }
 
@@ -602,7 +592,7 @@ void hb_frame_buffer_blank_stride(hb_buffer_t * buf)
     uint8_t * data;
     int       pp, yy, width, height, stride, height_stride;
 
-    for (pp = 0; pp < 4; pp++)
+    for (pp = 0; pp <= buf->f.max_plane; pp++)
     {
         data          = buf->plane[pp].data;
         width         = buf->plane[pp].width;
@@ -632,7 +622,7 @@ void hb_frame_buffer_mirror_stride(hb_buffer_t * buf)
     int       pp, ii, yy, width, height, stride, height_stride;
     int       pos, margin, margin_front, margin_back;
 
-    for (pp = 0; pp < 4; pp++)
+    for (pp = 0; pp <= buf->f.max_plane; pp++)
     {
         data          = buf->plane[pp].data;
         width         = buf->plane[pp].width;
@@ -674,26 +664,29 @@ void hb_frame_buffer_mirror_stride(hb_buffer_t * buf)
 // with dimensions width x height.
 void hb_video_buffer_realloc( hb_buffer_t * buf, int width, int height )
 {
-    const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(buf->f.fmt);
-    int p;
-    uint8_t has_plane[4] = {0,};
+    const AVPixFmtDescriptor * desc = av_pix_fmt_desc_get(buf->f.fmt);
+    uint8_t                    has_plane[4] = {0,};
+    int                        ii, pp;
 
     if (desc == NULL)
     {
         return;
     }
-    for( p = 0; p < 4; p++ )
-    {
-        has_plane[desc->comp[p].plane] = 1;
-    }
 
+    buf->f.max_plane = 0;
     int size = 0;
-    for( p = 0; p < 4; p++ )
+    for (ii = 0; ii < desc->nb_components; ii++)
     {
-        if ( has_plane[p] )
+        pp = desc->comp[ii].plane;
+        if (pp > buf->f.max_plane)
         {
-            size += hb_image_stride( buf->f.fmt, width, p ) * 
-                    hb_image_height_stride( buf->f.fmt, height, p );
+            buf->f.max_plane = pp;
+        }
+        if (!has_plane[pp])
+        {
+            has_plane[pp] = 1;
+            size += hb_image_stride(buf->f.fmt, width, pp) *
+                    hb_image_height_stride(buf->f.fmt, height, pp );
         }
     }
 
@@ -703,7 +696,7 @@ void hb_video_buffer_realloc( hb_buffer_t * buf, int width, int height )
     buf->f.height = height;
     buf->size = size;
 
-    hb_buffer_init_planes_internal( buf, has_plane );
+    hb_buffer_init_planes(buf);
 }
 
 // this routine 'moves' data from src to dst by interchanging 'data',
@@ -729,9 +722,18 @@ void hb_buffer_close( hb_buffer_t ** _b )
 
     while( b )
     {
-#ifdef USE_QSV
+#if HB_PROJECT_FEATURE_QSV
         // Reclaim QSV resources before dropping the buffer.
         // when decoding without QSV, the QSV atom will be NULL.
+        if(b->qsv_details.frame)
+        {
+            mfxFrameSurface1 *surface = (mfxFrameSurface1*)b->qsv_details.frame->data[3];
+            if(surface)
+            {
+                hb_qsv_release_surface_from_pool(surface->Data.MemId);
+                b->qsv_details.frame->data[3] = 0;
+            }
+        }
         if (b->qsv_details.qsv_atom != NULL && b->qsv_details.ctx != NULL)
         {
             hb_qsv_stage *stage = hb_qsv_get_last_stage(b->qsv_details.qsv_atom);
@@ -776,10 +778,10 @@ void hb_buffer_close( hb_buffer_t ** _b )
             continue;
         }
         // either the pool is full or this size doesn't use a pool
-        // free the buf 
+        // free the buf
         if( b->data )
         {
-            free(b->data);
+            av_free(b->data);
             hb_lock(buffers.lock);
             buffers.allocated -= b->alloc;
             hb_unlock(buffers.lock);
@@ -793,42 +795,40 @@ void hb_buffer_close( hb_buffer_t ** _b )
 
 hb_image_t * hb_image_init(int pix_fmt, int width, int height)
 {
-    const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(pix_fmt);
-    int p;
-    uint8_t has_plane[4] = {0,};
+    const AVPixFmtDescriptor * desc = av_pix_fmt_desc_get(pix_fmt);
+    uint8_t                    has_plane[4] = {0,};
+    int                        ii, pp;
 
     if (desc == NULL)
     {
         return NULL;
     }
-    for (p = 0; p < 4; p++)
-    {
-        has_plane[desc->comp[p].plane] = 1;
-    }
-
-    int size = 0;
-    for (p = 0; p < 4; p++)
-    {
-        if (has_plane[p])
-        {
-            size += hb_image_stride( pix_fmt, width, p ) * 
-                    hb_image_height_stride( pix_fmt, height, p );
-        }
-    }
 
     hb_image_t *image = calloc(1, sizeof(hb_image_t));
     if (image == NULL)
     {
         return NULL;
     }
-#if defined( SYS_DARWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || defined( SYS_MINGW )
-    image->data  = malloc(size);
-#elif defined( SYS_CYGWIN )
-    /* FIXME */
-    image->data  = malloc(size + 17);
-#else
-    image->data  = memalign(16, size);
-#endif
+
+    int size = 0;
+    for (ii = 0; ii < desc->nb_components; ii++)
+    {
+        // For non-planar formats, comp[ii].plane can contain the
+        // same value for multiple comp.
+        pp = desc->comp[ii].plane;
+        if (pp > image->max_plane)
+        {
+            image->max_plane = pp;
+        }
+        if (!has_plane[pp])
+        {
+            has_plane[pp] = 1;
+            size += hb_image_stride( pix_fmt, width, pp ) *
+                    hb_image_height_stride( pix_fmt, height, pp );
+        }
+    }
+
+    image->data  = av_malloc(size);
     if (image->data == NULL)
     {
         free(image);
@@ -839,21 +839,18 @@ hb_image_t * hb_image_init(int pix_fmt, int width, int height)
     image->height = height;
     memset(image->data, 0, size);
 
-    uint8_t * plane = image->data;
-    for (p = 0; p < 4; p++)
+    uint8_t * data = image->data;
+    for (pp = 0; pp <= image->max_plane; pp++)
     {
-        if (has_plane[p])
-        {
-            image->plane[p].data = plane;
-            image->plane[p].stride = hb_image_stride(pix_fmt, width, p );
-            image->plane[p].height_stride =
-                                    hb_image_height_stride(pix_fmt, height, p );
-            image->plane[p].width  = hb_image_width(pix_fmt, width, p );
-            image->plane[p].height = hb_image_height(pix_fmt, height, p );
-            image->plane[p].size   =
-                        image->plane[p].stride * image->plane[p].height_stride;
-            plane += image->plane[p].size;
-        }
+        image->plane[pp].data   = data;
+        image->plane[pp].stride = hb_image_stride(pix_fmt, width, pp);
+        image->plane[pp].height_stride =
+                                hb_image_height_stride(pix_fmt, height, pp);
+        image->plane[pp].width  = hb_image_width(pix_fmt, width, pp);
+        image->plane[pp].height = hb_image_height(pix_fmt, height, pp);
+        image->plane[pp].size   = image->plane[pp].stride *
+                                  image->plane[pp].height_stride;
+        data                   += image->plane[pp].size;
     }
     return image;
 }
@@ -862,14 +859,7 @@ hb_image_t * hb_buffer_to_image(hb_buffer_t *buf)
 {
     hb_image_t *image = calloc(1, sizeof(hb_image_t));
 
-#if defined( SYS_DARWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || defined( SYS_MINGW )
-    image->data  = malloc( buf->size );
-#elif defined( SYS_CYGWIN )
-    /* FIXME */
-    image->data  = malloc( buf->size + 17 );
-#else
-    image->data  = memalign( 16, buf->size );
-#endif
+    image->data  = av_malloc( buf->size );
     if (image->data == NULL)
     {
         free(image);
@@ -883,7 +873,7 @@ hb_image_t * hb_buffer_to_image(hb_buffer_t *buf)
 
     int p;
     uint8_t *data = image->data;
-    for (p = 0; p < 4; p++)
+    for (p = 0; p <= buf->f.max_plane; p++)
     {
         image->plane[p].data = data;
         image->plane[p].width = buf->plane[p].width;
@@ -904,7 +894,7 @@ void hb_image_close(hb_image_t **_image)
     hb_image_t * image = *_image;
     if (image != NULL)
     {
-        free(image->data);
+        av_free(image->data);
         free(image);
         *_image = NULL;
     }
@@ -1219,7 +1209,7 @@ void hb_fifo_push_head( hb_fifo_t * f, hb_buffer_t * b )
     if( f->size > 0 )
     {
         tmp->next = f->first;
-    } 
+    }
     else
     {
         f->last = tmp;
