@@ -9,8 +9,8 @@
 
 #include <ctype.h>
 #include <stdio.h>
-#include "hb.h"
-#include "hb_dict.h"
+#include "handbrake/handbrake.h"
+#include "handbrake/hb_dict.h"
 
 hb_value_type_t hb_value_type(const hb_value_t *value)
 {
@@ -469,6 +469,11 @@ void hb_dict_set(hb_dict_t * dict, const char *key, hb_value_t *value)
     json_object_set_new(dict, key, value);
 }
 
+void hb_dict_merge(hb_dict_t * dict, hb_dict_t *value)
+{
+    json_object_update(dict, value);
+}
+
 void hb_dict_case_set(hb_dict_t * dict, const char *key, hb_value_t *value)
 {
     char * lower = makelower(key);
@@ -826,7 +831,7 @@ hb_dict_t * hb_encopts_to_dict(const char * encopts, int encoder)
                 // x264 has multiple names for some options
                 if (encoder & HB_VCODEC_X264_MASK)
                     name = hb_x264_encopt_name(name);
-#ifdef USE_X265
+#if HB_PROJECT_FEATURE_X265
                 // x265 has multiple names for some options
                 if (encoder & HB_VCODEC_X265_MASK)
                     name = hb_x265_encopt_name(name);
