@@ -12,8 +12,10 @@
 #include "libavutil/avstring.h"
 #include "libavutil/intreadwrite.h"
 
-#include "hb.h"
-#include "lang.h"
+#include "handbrake/handbrake.h"
+#include "handbrake/ssautil.h"
+#include "handbrake/lang.h"
+#include "handbrake/hbffmpeg.h"
 
 struct hb_mux_data_s
 {
@@ -33,7 +35,8 @@ struct hb_mux_data_s
     int64_t  prev_chapter_tc;
     int16_t  current_chapter;
 
-    AVBSFContext             * bitstream_context;
+    AVBSFContext            * bitstream_context;
+    hb_tx3g_style_context_t * tx3g;
 };
 
 struct hb_mux_object_s
@@ -68,6 +71,7 @@ enum
 {
     META_MUX_MP4,
     META_MUX_MKV,
+    META_MUX_WEBM,
     META_MUX_LAST
 };
 
@@ -96,6 +100,7 @@ static char* lookup_lang_code(int mux, char *iso639_2)
             out = iso639_2;
             break;
         case HB_MUX_AV_MKV:
+        case HB_MUX_AV_WEBM: // webm is a subset of mkv
             // MKV lang codes should be ISO-639-2B if it exists,
             // else ISO-639-2
             lang =  lang_for_code2( iso639_2 );
@@ -162,6 +167,15 @@ static int avformatInit( hb_mux_object_t * m )
             meta_mux = META_MUX_MKV;
             break;
 
+        case HB_MUX_AV_WEBM:
+            // libavformat is essentially hard coded such that it only
+            // works with a timebase of 1/1000
+            m->time_base.num = 1;
+            m->time_base.den = 1000;
+            muxer_name = "webm";
+            meta_mux = META_MUX_WEBM;
+            break;
+
         default:
         {
             hb_error("Invalid Mux %x", job->mux);
@@ -660,7 +674,7 @@ static int avformatInit( hb_mux_object_t * m )
             track->st->codecpar->channel_layout = hb_ff_mixdown_xlat(audio->config.out.mixdown, NULL);
         }
 
-        char *name;
+        const char *name;
         if (audio->config.out.name == NULL)
         {
             switch (track->st->codecpar->channels)
@@ -688,7 +702,7 @@ static int avformatInit( hb_mux_object_t * m )
         {
             // Some software (MPC, mediainfo) use hdlr description
             // for track title
-            av_dict_set(&track->st->metadata, "handler", name, 0);
+            av_dict_set(&track->st->metadata, "handler_name", name, 0);
         }
     }
 
@@ -842,13 +856,16 @@ static int avformatInit( hb_mux_object_t * m )
             case CC608SUB:
             case CC708SUB:
             case TX3GSUB:
-            case SRTSUB:
             case UTF8SUB:
             case SSASUB:
+            case IMPORTSRT:
+            case IMPORTSSA:
             {
                 if (job->mux == HB_MUX_AV_MP4)
                 {
                     track->st->codecpar->codec_id = AV_CODEC_ID_MOV_TEXT;
+                    track->tx3g = hb_tx3g_style_init(
+                                job->height, (char*)subtitle->extradata);
                 }
                 else
                 {
@@ -944,6 +961,19 @@ static int avformatInit( hb_mux_object_t * m )
         {
             av_dict_set(&track->st->metadata, "language", lang, 0);
         }
+        if (subtitle->config.name != NULL && subtitle->config.name[0] != 0)
+        {
+            // Set subtitle track title
+            av_dict_set(&track->st->metadata, "title",
+                        subtitle->config.name, 0);
+            if (job->mux == HB_MUX_AV_MP4)
+            {
+                // Some software (MPC, mediainfo) use hdlr description
+                // for track title
+                av_dict_set(&track->st->metadata, "handler_name",
+                            subtitle->config.name, 0);
+            }
+        }
     }
 
     if (need_fonts)
@@ -1128,7 +1158,7 @@ static int add_chapter(hb_mux_object_t *m, int64_t start, int64_t end, char * ti
     chap->id = nchap;
     chap->time_base = m->time_base;
     // libav does not currently have a good way to deal with chapters and
-    // delayed stream timestamps.  It makes no corrections to the chapter 
+    // delayed stream timestamps.  It makes no corrections to the chapter
     // track.  A patch to libav would touch a lot of things, so for now,
     // work around the issue here.
     chap->start = start;
@@ -1179,14 +1209,15 @@ static int avformatMux(hb_mux_object_t *m, hb_mux_data_t *track, hb_buffer_t *bu
         return 0;
     }
 
-    if (track->type == MUX_TYPE_VIDEO && (job->mux & HB_MUX_MASK_MKV) &&
+    if (track->type == MUX_TYPE_VIDEO &&
+        (job->mux & (HB_MUX_MASK_MKV | HB_MUX_MASK_WEBM)) &&
         buf->s.renderOffset < 0)
     {
         // libav matroska muxer doesn't write dts to the output, but
         // if it sees a negative dts, it applies an offset to both pts
         // and dts to make it positive.  This offset breaks chapter
         // start times and A/V sync.  libav also requires that dts is
-        // "monotically increasing", which means it last_dts <= next_dts.
+        // "monotonically increasing", which means it last_dts <= next_dts.
         // It also uses dts to determine track interleaving, so we need
         // to provide some reasonable dts value.
         // So when renderOffset < 0, set to 0 for mkv.
@@ -1353,8 +1384,8 @@ static int avformatMux(hb_mux_object_t *m, hb_mux_data_t *track, hb_buffer_t *bu
                      * creating style atoms for them.
                      */
                     hb_muxmp4_process_subtitle_style(
-                        job->height, buf->data, &buffer,
-                        &styleatom, &stylesize );
+                        track->tx3g, buf->data, &buffer,
+                        &styleatom, &stylesize);
 
                     if (buffer != NULL)
                     {
@@ -1462,6 +1493,10 @@ static int avformatEnd(hb_mux_object_t *m)
         {
             av_bsf_free(&m->tracks[ii]->bitstream_context);
         }
+        if (m->tracks[ii]->tx3g)
+        {
+            hb_tx3g_style_close(&m->tracks[ii]->tx3g);
+        }
     }
 
     if (job->chapter_markers)
