@@ -7,6 +7,8 @@
    For full terms see the file COPYING file or visit http://www.gnu.org/licenses/gpl-2.0.html
  */
 
+#include "handbrake/project.h"
+
 #ifdef SYS_MINGW
 #define _WIN32_WINNT 0x600
 #endif
@@ -70,6 +72,9 @@
 #include <linux/cdrom.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
+#if HB_PROJECT_FEATURE_QSV
+#include <libdrm/drm.h>
+#endif
 #elif defined( SYS_OPENBSD )
 #include <sys/dvdio.h>
 #include <fcntl.h>
@@ -83,7 +88,7 @@
 #include <stddef.h>
 #include <unistd.h>
 
-#include "hb.h"
+#include "handbrake/handbrake.h"
 #include "libavutil/cpu.h"
 
 /************************************************************************
@@ -256,8 +261,6 @@ const char* hb_get_cpu_platform_name()
 {
     switch (hb_cpu_info.platform)
     {
-        // Intel 64 and IA-32 Architectures Software Developer's Manual, Vol. 3C
-        // Table 35-1: CPUID Signature Values of DisplayFamily_DisplayModel
         case HB_CPU_PLATFORM_INTEL_BNL:
             return "Intel microarchitecture Bonnell";
         case HB_CPU_PLATFORM_INTEL_SNB:
@@ -276,6 +279,8 @@ const char* hb_get_cpu_platform_name()
             return "Intel microarchitecture Airmont";
         case HB_CPU_PLATFORM_INTEL_KBL:
             return "Intel microarchitecture Kaby Lake";
+        case HB_CPU_PLATFORM_INTEL_ICL:
+            return "Intel microarchitecture Ice Lake";
         default:
             return NULL;
     }
@@ -314,8 +319,8 @@ static void init_cpu_info()
         family = ((eax >> 8) & 0xf) + ((eax >> 20) & 0xff);
         model  = ((eax >> 4) & 0xf) + ((eax >> 12) & 0xf0);
 
-        // Intel 64 and IA-32 Architectures Software Developer's Manual, Vol. 3C
-        // Table 35-1: CPUID Signature Values of DisplayFamily_DisplayModel
+        // Intel 64 and IA-32 Architectures Software Developer's Manual, Volume 4/January 2019
+        // Table 2-1. CPUID Signature Values of DisplayFamily_DisplayModel
         switch (family)
         {
             case 0x06:
@@ -366,6 +371,8 @@ static void init_cpu_info()
                     case 0x9E:
                         hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_KBL;
                         break;
+                    case 0x7E:
+                        hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_ICL;
                     default:
                         break;
                 }
@@ -566,7 +573,7 @@ void hb_get_user_config_directory( char path[512] )
         return;
     }
 #elif defined( __APPLE__ )
-    if (osx_get_user_config_directory(path) == 0)
+    if (macOS_get_user_config_directory(path) == 0)
     {
         return;
     }
@@ -859,6 +866,10 @@ static void attribute_align_thread hb_thread_func( void * _t )
     pthread_setschedparam( pthread_self(), SCHED_OTHER, &param );
 #endif
 
+#if defined( SYS_DARWIN )
+    pthread_setname_np( t->name );
+#endif
+
 #if defined( SYS_BEOS )
     signal( SIGINT, SIG_IGN );
 #endif
@@ -1346,7 +1357,7 @@ void hb_system_sleep_private_disable(void *opaque)
         hb_error("hb_system_sleep: opaque is NULL");
         return;
     }
-    
+
     IOPMAssertionID *assertionID = (IOPMAssertionID*)opaque;
     if (*assertionID != -1)
     {
@@ -1405,3 +1416,291 @@ int hb_dlclose(void *h)
 #endif
 }
 
+size_t hb_getline(char ** lineptr, size_t * n, FILE * fp)
+{
+#ifdef SYS_MINGW
+    char   * bufptr = NULL;
+    char   * p      = bufptr;
+    size_t   size;
+    int      c;
+
+    if (lineptr == NULL)
+    {
+        return -1;
+    }
+    if (fp == NULL)
+    {
+        return -1;
+    }
+    if (n == NULL)
+    {
+        return -1;
+    }
+    bufptr = *lineptr;
+    size   = *n;
+
+    c = fgetc(fp);
+    if (c == EOF)
+    {
+        return -1;
+    }
+    if (bufptr == NULL)
+    {
+        bufptr = malloc(128);
+        if (bufptr == NULL)
+        {
+            return -1;
+        }
+        size = 128;
+    }
+    p = bufptr;
+    while (c != EOF)
+    {
+        if ((p - bufptr) >= (size - 1))
+        {
+            char * tmp;
+            size = size + 128;
+            tmp = realloc(bufptr, size);
+            if (tmp == NULL)
+            {
+                free(bufptr);
+                return -1;
+            }
+            p = tmp + (p - bufptr);
+            bufptr = tmp;
+        }
+        *p++ = c;
+        if (c == '\n')
+        {
+            break;
+        }
+        c = fgetc(fp);
+    }
+
+    *p++ = '\0';
+    *lineptr = bufptr;
+    *n = size;
+
+    return p - bufptr - 1;
+#else
+    return getline(lineptr, n, fp);
+#endif
+}
+
+char * hb_strndup(const char * src, size_t len)
+{
+#ifdef SYS_MINGW
+    char * result, * end;
+
+    if (src == NULL)
+    {
+        return NULL;
+    }
+
+    end = memchr(src, 0, len);
+    if (end != NULL)
+    {
+        len = end - src;
+    }
+
+    result = malloc(len + 1);
+    if (result == NULL)
+    {
+        return NULL;
+    }
+    memcpy(result, src, len);
+    result[len] = 0;
+
+    return result;
+#else
+    return strndup(src, len);
+#endif
+}
+
+#if HB_PROJECT_FEATURE_QSV
+#ifdef SYS_LINUX
+
+#define MAX_NODES             16
+#define DRI_RENDER_NODE_START 128
+#define DRI_RENDER_NODE_LAST  (DRI_RENDER_NODE_START + MAX_NODES - 1)
+#define DRI_CARD_NODE_START   0
+#define DRI_CARD_NODE_LAST    (DRI_CARD_NODE_START + MAX_NODES - 1)
+
+const char* DRI_PATH = "/dev/dri/";
+const char* DRI_NODE_RENDER = "renderD";
+const char* DRI_NODE_CARD = "card";
+
+static int try_adapter(const char * name, const char * dir,
+                const char * prefix, int node_start, int node_last)
+{
+    int             node;
+    int             len        = strlen(name);
+    char          * driverName = malloc(len + 1);
+    drm_version_t   version = {};
+
+    version.name_len = len + 1;
+    version.name     = driverName;
+    for (node = node_start; node <= node_last; node++)
+    {
+        char * adapter = hb_strdup_printf("%s%s%d", dir, prefix, node);
+        int    fd      = open(adapter, O_RDWR);
+
+        free(adapter);
+        if (fd < 0)
+        {
+            continue;
+        }
+
+        if (!ioctl(fd, DRM_IOCTL_VERSION, &version) &&
+            version.name_len == len && !strncmp(driverName, name, len))
+        {
+            free(driverName);
+            return fd;
+        }
+        close(fd);
+    }
+
+    free(driverName);
+    return -1;
+}
+
+static int open_adapter(const char * name)
+{
+    int fd = try_adapter(name, DRI_PATH, DRI_NODE_RENDER,
+                         DRI_RENDER_NODE_START, DRI_RENDER_NODE_LAST);
+    if (fd < 0)
+    {
+        fd = try_adapter(name, DRI_PATH, DRI_NODE_CARD,
+                         DRI_CARD_NODE_START, DRI_CARD_NODE_LAST);
+    }
+    return fd;
+}
+
+static int try_va_interface(hb_display_t * hbDisplay,
+                            const char * interface_name)
+{
+    if (interface_name != NULL)
+    {
+        setenv("LIBVA_DRIVER_NAME", interface_name, 1);
+    }
+
+    hbDisplay->vaDisplay = vaGetDisplayDRM(hbDisplay->vaFd);
+    if (hbDisplay->vaDisplay == NULL)
+    {
+        return -1;
+    }
+
+    int major = 0, minor = 0;
+    VAStatus vaRes = vaInitialize(hbDisplay->vaDisplay, &major, &minor);
+    if (vaRes != VA_STATUS_SUCCESS)
+    {
+        vaTerminate(hbDisplay->vaDisplay);
+        return -1;
+    }
+    hbDisplay->handle = hbDisplay->vaDisplay;
+    hbDisplay->mfxType = MFX_HANDLE_VA_DISPLAY;
+
+    return 0;
+}
+
+hb_display_t * hb_display_init(const char         *  driver_name,
+                               const char * const * interface_names)
+{
+    hb_display_t * hbDisplay = calloc(sizeof(hb_display_t), 1);
+    char         * env;
+    int            ii;
+
+    hbDisplay->vaDisplay = NULL;
+    hbDisplay->vaFd      = open_adapter(driver_name);
+    if (hbDisplay->vaFd < 0)
+    {
+        hb_deep_log( 3, "hb_va_display_init: no display found" );
+        free(hbDisplay);
+        return NULL;
+    }
+
+    if ((env = getenv("LIBVA_DRIVER_NAME")) != NULL)
+    {
+        // Use only environment if it's set
+        hb_log("hb_display_init: using VA driver '%s'", env);
+        if (try_va_interface(hbDisplay, NULL) != 0)
+        {
+            close(hbDisplay->vaFd);
+            free(hbDisplay);
+            return NULL;
+        }
+    }
+    else
+    {
+        // Try list of VA driver names
+        for (ii = 0; interface_names[ii] != NULL; ii++)
+        {
+            hb_log("hb_display_init: attempting VA driver '%s'",
+                   interface_names[ii]);
+            if (try_va_interface(hbDisplay, interface_names[ii]) == 0)
+            {
+                return hbDisplay;
+            }
+        }
+        // Try default
+        unsetenv("LIBVA_DRIVER_NAME");
+        hb_log("hb_display_init: attempting VA default driver");
+        if (try_va_interface(hbDisplay, NULL) != 0)
+        {
+            close(hbDisplay->vaFd);
+            free(hbDisplay);
+            return NULL;
+        }
+    }
+    return hbDisplay;
+}
+
+void hb_display_close(hb_display_t ** _d)
+{
+    hb_display_t * hbDisplay = *_d;
+
+    if (hbDisplay == NULL)
+    {
+        return;
+    }
+    if (hbDisplay->vaDisplay)
+    {
+        vaTerminate(hbDisplay->vaDisplay);
+    }
+    if (hbDisplay->vaFd >= 0)
+    {
+        close(hbDisplay->vaFd);
+    }
+    free(hbDisplay);
+
+    *_d = NULL;
+}
+
+#else // !SYS_LINUX
+
+hb_display_t * hb_display_init(const char         *  driver_name,
+                               const char * const * interface_names)
+{
+    return NULL;
+}
+
+void hb_display_close(hb_display_t ** _d)
+{
+    (void)_d;
+}
+
+#endif // SYS_LINUX
+#else // !HB_PROJECT_FEATURE_QSV
+
+hb_display_t * hb_display_init(const char         *  driver_name,
+                               const char * const * interface_names)
+{
+    return NULL;
+}
+
+void hb_display_close(hb_display_t ** _d)
+{
+    (void)_d;
+}
+
+#endif // HB_PROJECT_FEATURE_QSV
