@@ -7,8 +7,8 @@
    For full terms see the file COPYING file or visit http://www.gnu.org/licenses/gpl-2.0.html
  */
 
-#include "hb.h"
-#include "hbffmpeg.h"
+#include "handbrake/handbrake.h"
+#include "handbrake/hbffmpeg.h"
 
 typedef struct
 {
@@ -75,6 +75,7 @@ hb_thread_t * hb_scan_init( hb_handle_t * handle, volatile int * die,
 
     // Initialize scan state
     hb_state_t state;
+    hb_get_state2(handle, &state);
 #define p state.param.scanning
     state.state   = HB_STATE_SCANNING;
     p.title_cur   = 1;
@@ -294,12 +295,12 @@ static void ScanFunc( void * _data )
     }
     if (hb_list_count(data->title_set->list_title) > 0)
     {
-        strncpy(data->title_set->path, data->path, 1024);
-        data->title_set->path[1023] = 0;
+        data->title_set->path = strdup(data->path);
     }
     else
     {
-        data->title_set->path[0] = 0;
+        free((char*)data->title_set->path);
+        data->title_set->path = NULL;
     }
 
 finish:
@@ -734,7 +735,7 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
                     // There are 2 conditions we decode additional
                     // video frames for during scan.
                     // 1. We did not detect IDR frames, so the initial video
-                    //    frames may be corrupt.  We docode extra frames to
+                    //    frames may be corrupt.  We decode extra frames to
                     //    increase the probability of a complete preview frame
                     // 2. Some frames do not contain CC data, even though
                     //    CCs are present in the stream.  So we need to decode
@@ -1005,9 +1006,11 @@ skip_preview:
         {
             title->geometry.par = vid_info.geometry.par;
         }
+        title->pix_fmt = vid_info.pix_fmt;
         title->color_prim = vid_info.color_prim;
         title->color_transfer = vid_info.color_transfer;
         title->color_matrix = vid_info.color_matrix;
+        title->color_range = vid_info.color_range;
 
         title->video_decode_support = vid_info.video_decode_support;
 
@@ -1316,19 +1319,25 @@ static void LookForAudio(hb_scan_t *scan, hb_title_t * title, hb_buffer_t * b)
     if (audio->config.lang.attributes & HB_AUDIO_ATTR_VISUALLY_IMPAIRED)
     {
         strncat(audio->config.lang.description, " (Visually Impaired)",
-                sizeof(audio->config.lang.description) - 
+                sizeof(audio->config.lang.description) -
                 strlen(audio->config.lang.description) - 1);
     }
     if (audio->config.lang.attributes & HB_AUDIO_ATTR_COMMENTARY)
     {
         strncat(audio->config.lang.description, " (Director's Commentary 1)",
-                sizeof(audio->config.lang.description) - 
+                sizeof(audio->config.lang.description) -
                 strlen(audio->config.lang.description) - 1);
     }
     if (audio->config.lang.attributes & HB_AUDIO_ATTR_ALT_COMMENTARY)
     {
         strncat(audio->config.lang.description, " (Director's Commentary 2)",
-                sizeof(audio->config.lang.description) - 
+                sizeof(audio->config.lang.description) -
+                strlen(audio->config.lang.description) - 1);
+    }
+    if (audio->config.lang.attributes & HB_AUDIO_ATTR_SECONDARY)
+    {
+        strncat(audio->config.lang.description, " (Secondary)",
+                sizeof(audio->config.lang.description) -
                 strlen(audio->config.lang.description) - 1);
     }
 
@@ -1375,6 +1384,16 @@ static void LookForAudio(hb_scan_t *scan, hb_title_t * title, hb_buffer_t * b)
         }
     }
 
+    // Append input bitrate in kbps to the end of the description if greater than 1
+    // ffmpeg may report some audio bitrates as 1, not an issue
+    if (audio->config.in.bitrate > 1)
+    {
+        char in_bitrate_str[19];
+        snprintf(in_bitrate_str, 18, " (%d kbps)", audio->config.in.bitrate / 1000);
+        strncat(audio->config.lang.description, in_bitrate_str,
+                sizeof(audio->config.lang.description) - strlen(audio->config.lang.description) - 1);
+    }
+
     hb_log( "scan: audio 0x%x: %s, rate=%dHz, bitrate=%d %s", audio->id,
             info.name, audio->config.in.samplerate, audio->config.in.bitrate,
             audio->config.lang.description );
@@ -1422,6 +1441,7 @@ static void UpdateState1(hb_scan_t *scan, int title)
 {
     hb_state_t state;
 
+    hb_get_state2(scan->h, &state);
 #define p state.param.scanning
     /* Update the UI */
     state.state   = HB_STATE_SCANNING;
@@ -1442,6 +1462,7 @@ static void UpdateState2(hb_scan_t *scan, int title)
 {
     hb_state_t state;
 
+    hb_get_state2(scan->h, &state);
 #define p state.param.scanning
     /* Update the UI */
     state.state   = HB_STATE_SCANNING;
